import logging

from OBP_reliability_pillar.s3.s3_bucket_default_lock_enabled import s3_bucket_default_lock_enabled
from OBP_reliability_pillar.s3.s3_bucket_replication_enabled import s3_bucket_replication_enabled
from OBP_reliability_pillar.s3.s3_bucket_versioning_enabled import s3_bucket_versioning_enabled

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


#     list s3 buckets
def list_s3_buckets(self) -> list:
    """
    :return:
    """
    logger.info(" ---Inside utils :: list_s3_buckets")

    buckets = []

    client = self.session.client('s3')
    response = client.list_buckets()

    return response['Buckets']


# returns consolidated dynamodb compliance
def s3_compliance(self) -> list:
    """
    :param self:
    :return:
    """
    logger.info(" ---Inside s3 :: s3_compliance()")

    buckets = self.list_s3_buckets()

    response = [
        s3_bucket_replication_enabled(self, buckets),
        s3_bucket_versioning_enabled(self, buckets),
        s3_bucket_default_lock_enabled(self, buckets)
    ]

    return response



from OBP_reliability_pillar.rds.rds_automatic_minor_version_upgrade_enabled import *
# from OBP_reliability_pillar.rds.rds_backup_enabled import rds_backup_enabled
from OBP_reliability_pillar.rds.rds_enhanced_monitoring_enabled import *
from OBP_reliability_pillar.rds.rds_multi_az_support_enabled import *
from OBP_reliability_pillar.rds.rds_instance_deletion_protection_enabled import *

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# list rds instances
def list_rds_instances(self, regions) -> dict:
    """
    :param regions:
    :return:
    """
    logger.info(" ---Inside utils :: list_rds_instances()--- ")
    rds_instance_lst = {}

    for region in regions:
        client = self.session.client('rds', region_name=region)
        marker = ''
        while True:
            if marker == '':
                response = client.describe_db_instances(
                    MaxRecords=100
                )
            else:
                response = client.describe_db_instances(
                    MaxRecords=100,
                    Marker=marker
                )
            rds_instance_lst.setdefault(region, []).extend(response['DBInstances'])

            try:
                marker = response['Marker']
                if marker == '':
                    break
            except KeyError:
                break
    return rds_instance_lst


# returns consolidated dynamodb compliance
def rds_compliance(self, regions) -> list:
    """
    :param regions:
    :param self:
    :return:
    """
    logger.info(" ---Inside rds :: rds_compliance()")
    rds_instances = self.list_rds_instances(regions=regions)

    response = [
        rds_multi_az_support_enabled(self, rds_instances),
        rds_instance_deletion_protection_enabled(self, rds_instances),

        # Already covered in monitoring module, hence commenting here
        # rds_enhanced_monitoring_enabled(self),

        rds_automatic_minor_version_upgrade_enabled(self, rds_instances),
        # rds_backup_enabled(self),
    ]

    return response



import logging
import botocore

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# checks compliance.py for rds automatic minor version enabled
def rds_multi_az_support_enabled(self, rds_instances: dict) -> dict:
    """
    :param self:
    :param rds_instances:
    """
    logger.info(" ---Inside rds :: rds_multi_az_support_enabled()")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id3.78'
    compliance_type = "RDS multi az support enabled"
    description = "Checks if multi az support is enabled for RDS instances."
    resource_type = "RDS Instance"
    risk_level = 'Medium'

    # regions = self.session.get_available_regions('rds')

    for region, instances in rds_instances.items():
        for instance in instances:
            multi_az = instance['MultiAZ']
            if not multi_az:
                result = False
                failReason = "Multi az support is not enabled"
                offenders.append(region + ': ' + instance['DBInstanceIdentifier'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'ControlId': control_id,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level
    }



# import logging
#
# from botocore.exceptions import ClientError
#
# logging.basicConfig(level=logging.INFO)
# logger = logging.getLogger()
#
#
# # Checks compliance for rds backup enabled
# def rds_backup_enabled(self) -> dict:
#     """
#     :param self:
#     :return:
#     """
#     logger.info(" ---Inside rds :: rds_backup_enabled()")
#
#     result = True
#     failReason = ''
#     offenders = []
#     control_id = ''
#     compliance_type = "RDS instance backup enabled"
#     description = "Checks if RDS DB instances have backups enabled"
#     resource_type = "RDS Instance"
#     risk_level = 'High'
#
#     regions = self.session.get_available_regions('rds')
#
#     for region in regions:
#         try:
#             client = self.session.client('rds', region_name=region)
#             marker = ''
#             while True:
#                 if marker == '' or marker is None:
#                     response = client.describe_db_instances(
#                         MaxRecords=100
#                     )
#                 else:
#                     response = client.describe_db_instances(
#                         MaxRecords=100,
#                         Marker=marker
#                     )
#                 for instance in response['DBInstances']:
#                     retention_period = instance['BackupRetentionPeriod']
#                     if retention_period <= 0:
#                         result = False
#                         failReason = 'RDS Instance backup is not enabled'
#                         offenders.append(instance['DBInstanceIdentifier'])
#
#                 try:
#                     marker = response['Marker']
#                     if marker == '':
#                         break
#                 except KeyError:
#                     break
#         except ClientError as e:
#             logger.error('Something went wrong with the region {}: {}'.format(region,e))
#
#     return {
#         'Result': result,
#         'failReason': failReason,
#         'resource_type': resource_type,
#         'ControlId': control_id,
#         'Offenders': offenders,
#         'Compliance_type': compliance_type,
#         'Description': description,
#         'Risk Level': risk_level
#     }
#



import logging

from botocore.exceptions import ClientError

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# checks compliance for redshift cluster maintenance settings check
def redshift_cluster_maintenance_settings_check(self, redshift_clusters) -> dict:
    """
    :param redshift_clusters:
    :param self:
    :return:
    """
    logger.info(" ---Inside redshift :: redshift_cluster_maintenance_settings_check()")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id3.83'
    compliance_type = "Redshift cluster maintenance settings check"
    description = "Checks whether Amazon Redshift clusters have the specified maintenance settings"
    resource_type = "Redshift"
    risk_level = 'Medium'

    # regions = self.session.get_available_regions('redshift')

    for region, clusters in redshift_clusters.items():
        for cluster in clusters:
            version_upgrade = cluster['AllowVersionUpgrade']
            window = cluster['PreferredMaintenanceWindow']
            automated_snapshot_retention_period = cluster['AutomatedSnapshotRetentionPeriod']

            if not version_upgrade and automated_snapshot_retention_period > 0:
                result = False
                failReason = 'allowVersionUpgrade is not enabled'
                offenders.append(cluster['ClusterIdentifier'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'ControlId': control_id,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level
    }



import logging
from OBP_reliability_pillar.redshift.redshift_backup_enabled import redshift_backup_enabled
from OBP_reliability_pillar.redshift.redshift_cluster_maintenancesettings_check import \
    redshift_cluster_maintenance_settings_check

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# returns the list of redshift clusters
def list_redshift_clusters(self, regions) -> dict:
    """
    :param self:
    :param regions:
    :return:
    """
    logger.info(" ---Inside redshift :: list_redshift_clusters()--- ")
    redshift_clusters = {}

    for region in regions:
        client = self.session.client('redshift', region_name=region)
        marker = ''
        while True:
            if marker == '' or marker is None:
                response = client.describe_clusters()
            else:
                response = client.describe_clusters(
                    Marker=marker
                )
            redshift_clusters.setdefault(region, []).extend(response['Clusters'])

            try:
                marker = response['Marker']
                if marker == '':
                    break
            except KeyError:
                break

    return redshift_clusters


# returns consolidated dynamodb compliance
def redshift_compliance(self, regions) -> list:
    """
    :param regions:
    :param self:
    :return:
    """
    redshift_clusters = self.list_redshift_clusters(regions)
    logger.info(" ---Inside redshift :: redshift_compliance()")

    response = [
        redshift_backup_enabled(self, redshift_clusters),
        redshift_cluster_maintenance_settings_check(self, redshift_clusters),
    ]

    return response




class redshift:
    def __init__(self, session):
        """

        :param session:
        """
        self.session = session

    from .compliance import redshift_compliance, list_redshift_clusters



import logging

from botocore.exceptions import ClientError

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


def redshift_backup_enabled(self, redshift_clusters) -> dict:
    """
    :param redshift_clusters:
    :param self:
    :return:
    """
    logger.info(" ---Inside redshift :: redshift_backup_enabled()")

    result = True
    failReason = ''
    offenders = []
    control_id = 'Id3.80'
    compliance_type = "Redshift Backup Enabled"
    description = "Checks if backup is enabled on redshift cluster or not"
    resource_type = "Redshift"
    risk_level = 'Medium'

    # regions = self.session.get_available_regions('redshift')

    for region, clusters in redshift_clusters.items():
        for cluster in clusters:
            retention_period = cluster['AutomatedSnapshotRetentionPeriod']
            if retention_period <= 0:
                result = False
                failReason = "Redshift backup is not enabled"
                offenders.append(region + ': ' + cluster['ClusterIdentifier'])

    return {
        'Result': result,
        'failReason': failReason,
        'resource_type': resource_type,
        'ControlId': control_id,
        'Offenders': offenders,
        'Compliance_type': compliance_type,
        'Description': description,
        'Risk Level': risk_level
    }




class rds:
    def __init__(self, session):
        """

        :param session:
        """
        self.session = session

    from .compliance import rds_compliance, list_rds_instances, rds_instance_deletion_protection_enabled, \
        rds_automatic_minor_version_upgrade_enabled, rds_multi_az_support_enabled, rds_enhanced_monitoring_enabled




class s3:
    def __init__(self, session):
        """

        :param session:
        """
        self.session = session

    from .compliance import s3_compliance, list_s3_buckets, s3_bucket_versioning_enabled,\
        s3_bucket_default_lock_enabled, s3_bucket_replication_enabled



import logging
from OBP_reliability_pillar.cloudwatch.alarm_action_check import alarm_action_check

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()


# checks compliance.py for cloudwatch
def cloudwatch_compliance(self, regions) -> dict:
    """
    :param self:
    :param regions:
    :return:
    """
    logger.info(" ---Inside cloudwatch_compliance()")
    response = [
        alarm_action_check(self, regions)
    ]

    return response
